; ##############################################################################
; ##############################################################################
; ##############################################################################
; WDTCSR  Watchdog Timer Control Register				
; WDIF WDIE WDP3 WDCE WDE WDP2 WDP1 WDP0				
; WDP3 WDP2 WDP1 WDP0 Time								
;	0 	0 	0 	0 	2K 	(2048) 	cycles 16 ms			
;	0 	0 	0 	1 	4K 	(4096) 	cycles 32 ms			
;	0 	0 	1 	0 	8K 	(8192) 	cycles 64 ms			
;	0 	0 	1 	1 	16K (16384) cycles 0.125 s			
;	0 	1 	0 	0 	32K (32768) cycles 0.25 s			
; 	0 	1 	0 	1 	64K (65536) cycles 0.5 s			
; *	0 	1 	1 	0 	128K (131072)cycles 1.0 s			
;	0 	1 	1 	1 	256K (262144)cycles 2.0 s			
;	1 	0 	0 	0 	512K (524288)cycles 4.0 s			
;	1 	0 	0 	1 	1024K(1048576)cycles 8.0 s			
; Wenn WDIE=1 dann wird zuerst die WDR-ISR aufgerufen 	
; und dann das System neu gestartet						
WDR_INITIALISIERUNG:
	WDR
	LDS TEMP, WDTCSR
	ori TEMP, (1<<WDCE) | (1<<WDE)
	STS WDTCSR, TEMP
	ldi temp, (1<<WDE)| (1<<WDIE)|(0<<WDP3)|(1<<WDP2)|(1<<WDP1)|(0<<WDP0)
	STS WDTCSR, TEMP
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; Diese Routine wird bei WDR-Zeitberschreitung aufgerufen, hier knnte man		
; sich z.B. den Fehler anzeigen lasen, oder DATEN in den EEPROM schreiben		
WDT:
	WDR
	CLI											; INTERRUPTS sperren	
	cbi (LED_ROT_PORT),LED_ROT_PIN_NR			; ROT  an	
	call wait_100ms
	sbi (LED_ROT_PORT),LED_ROT_PIN_NR			; ROT  aus	
	call wait_100ms
	rjmp WDT
reti
; ##############################################################################
; ##############################################################################
; ##############################################################################
SRAM_CLEAR:
	ldi ZL, LOW (SRAM_START)
	ldi ZH, HIGH(SRAM_START)
	ldi temp,0
SRAM_CLEAR_w:
	ST z+,temp
	cpi ZH, HIGH(RAMEND-2)	; Vergleich?		
	brlo SRAM_CLEAR_w		; kleiner => SPRUNG	
	cpi ZL, LOW(RAMEND-2)	; Vergleich?		
	brlo SRAM_CLEAR_w		; kleiner => SPRUNG	
	breq SRAM_CLEAR_w		; gleich  => SPRUNG	
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
STARTWERTE:
	; NULL						
	clr NULL
	; EINS						
	ldi temp,1
	mov EINS,temp
	; VOLL						
	ldi temp,255
	mov VOLL,temp
	; SRAM CHECK				
	ldi temp,'S'
	STS(adr_SRAM_CHECK),temp
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; PORT CLEAR																	
PORT_CLEAR:
	ldi temp, 0b00000000	
	out DDRA, temp
	out DDRB, temp
	out DDRC, temp
	out DDRD, temp
	out PORTA,temp				
	out PORTB,temp				
	out PORTC,temp				
	out PORTD,temp	
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
REGISTER_CLEAR:
	clr R0
	clr R1
	clr R2
	clr R3
	clr R4
	clr R5
	clr R6
	clr R7
	clr R8
	clr R9
	clr R10
	clr R11
	clr R12
	clr R13
	clr R14
	clr R15
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
PROGRAMM_CHECK:
	; NULL		
	tst NULL
	brne PROGRAMM_CHECK_ERROR
	; EINS		
	mov temp,EINS
	cpi temp,1
	brne PROGRAMM_CHECK_ERROR
	; VOLL		
	mov temp,VOLL
	cpi temp,255
	brne PROGRAMM_CHECK_ERROR
	; SRAM CHECK
	LDS temp,(adr_SRAM_CHECK)
	cpi temp,'S'
	brne PROGRAMM_CHECK_ERROR
	; IDE-immer Ausgang					
	sbis (IDE_WR_DDR),(IDE_WR_PIN_NR)	; springe, wenn ...	
	rjmp PROGRAMM_CHECK_ERROR_IDE
	sbis (IDE_RD_DDR),(IDE_RD_PIN_NR)	; springe, wenn ...	
	rjmp PROGRAMM_CHECK_ERROR_IDE
	sbis (IDE_A0_DDR),(IDE_A0_PIN_NR)	; springe, wenn ...	
	rjmp PROGRAMM_CHECK_ERROR_IDE
	sbis (IDE_A1_DDR),(IDE_A1_PIN_NR)	; springe, wenn ...	
	rjmp PROGRAMM_CHECK_ERROR_IDE
	sbis (IDE_A2_DDR),(IDE_A2_PIN_NR)	; springe, wenn ...	
	rjmp PROGRAMM_CHECK_ERROR_IDE
	sbis (IDE_CS0_DDR),(IDE_CS0_PIN_NR)	; springe, wenn ...	
	rjmp PROGRAMM_CHECK_ERROR_IDE
	sbis (IDE_CS1_DDR),(IDE_CS1_PIN_NR)	; springe, wenn ...	
	rjmp PROGRAMM_CHECK_ERROR_IDE
	; IDE-WR-RD immer HIGH				
	sbis (IDE_WR_PIN),(IDE_WR_PIN_NR)	; springe, wenn ...	
	rjmp PROGRAMM_CHECK_ERROR_IDE
	sbis (IDE_RD_PIN),(IDE_RD_PIN_NR)	; springe, wenn ...	
	rjmp PROGRAMM_CHECK_ERROR_IDE
	; IDE-DATA immer EINGANG			
	in temp,(IDE_DATA0_DDR)
	cpi temp,0x00
	brne PROGRAMM_CHECK_ERROR_IDE
	in temp,(IDE_DATA1_DDR)
	cpi temp,0x00
	brne PROGRAMM_CHECK_ERROR_IDE
	; IDE-DATA immer PULL-UP on			
	in temp,(IDE_DATA0_PORT)
	cpi temp,0xFF
	brne PROGRAMM_CHECK_ERROR_IDE
	in temp,(IDE_DATA1_PORT)
	cpi temp,0xFF
	brne PROGRAMM_CHECK_ERROR_IDE
	ret
;-------------------------------------------------------------------------------
PROGRAMM_CHECK_ERROR:
	CLI											; INTERRUPTS sperren	
	call wait_100ms
	cbi (LED_ROT_PORT),(LED_ROT_PIN_NR)			; ROT  an	
	call wait_100ms
	sbi (LED_ROT_PORT),(LED_ROT_PIN_NR)			; ROT  aus	
	rjmp PROGRAMM_CHECK_ERROR
;-------------------------------------------------------------------------------
PROGRAMM_CHECK_ERROR_IDE:
	CLI											; INTERRUPTS sperren	
	call TASTATUR_ABFRAGE
	call wait_100ms
	cbi (LED_ROT_PORT),(LED_ROT_PIN_NR)			; ROT  an	
	call wait_100ms
	sbi (LED_ROT_PORT),(LED_ROT_PIN_NR)			; ROT  aus	
	call LCD_CLEAR
	call LCD_TEXT_IDE
	call LCD_SPACE
	call LCD_I
	call LCD_SCHRAEGSTRICH
	call LCD_O
	call WAIT_100ms
	rjmp PROGRAMM_CHECK_ERROR_IDE
; ##############################################################################
; ##############################################################################
; ##############################################################################
LED_INITIALISIERUNG:
	sbi (LED_ROT_DDR)  ,(LED_ROT_PIN_NR)		; ROT		
;-------------------------------------------------------------------------------
	cbi (LED_ROT_PORT)  ,(LED_ROT_PIN_NR)		; ROT  an	
	call WAIT_20ms
	sbi (LED_ROT_PORT)  ,(LED_ROT_PIN_NR)		; ROT  aus	
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
TASTATUR_ABFRAGE:
	WDR
	clr temp
	cbi (TASTE_A_DDR),(TASTE_A_PIN_NR)	; auf Eingang					
	cbi (TASTE_B_DDR),(TASTE_B_PIN_NR)
	cbi (TASTE_C_DDR),(TASTE_C_PIN_NR)
	cbi (TASTE_D_DDR),(TASTE_D_PIN_NR)
	cbi (TASTE_E_DDR),(TASTE_E_PIN_NR)
;-------------------------------------------------------------------------------
	sbi (TASTE_A_PORT),(TASTE_A_PIN_NR)	; Pull-Up on					
	sbi (TASTE_B_PORT),(TASTE_B_PIN_NR)
	sbi (TASTE_C_PORT),(TASTE_C_PIN_NR)
	sbi (TASTE_D_PORT),(TASTE_D_PIN_NR)
	sbi (TASTE_E_PORT),(TASTE_E_PIN_NR)
	call WAIT_10us
;-------------------------------------------------------------------------------
	; Taste "A"gedrckt ?		
	sbis (TASTE_A_PIN),(TASTE_A_PIN_NR)	; springe, wenn BIT(PIN) high	
	SBR temp,(TASTE_A)
	; Taste "B"gedrckt ?		
	sbis (TASTE_B_PIN),(TASTE_B_PIN_NR)	; springe, wenn BIT(PIN) high	
	SBR temp,(TASTE_B)
	; Taste "C"gedrckt ?		
	sbis (TASTE_C_PIN),(TASTE_C_PIN_NR)	; springe, wenn BIT(PIN) high	
	SBR temp,(TASTE_C)
	; Taste "D"gedrckt ?		
	sbis (TASTE_D_PIN),(TASTE_D_PIN_NR)	; springe, wenn BIT(PIN) high	
	SBR temp,(TASTE_D)
	; Taste "E"gedrckt ?		
	sbis (TASTE_E_PIN),(TASTE_E_PIN_NR)	; springe, wenn BIT(PIN) high	
	SBR temp,(TASTE_E)
	STS(adr_TASTE),temp
;-------------------------------------------------------------------------------
	sbi (TASTE_A_DDR),(TASTE_A_PIN_NR)	; auf AUSGANG					
	sbi (TASTE_B_DDR),(TASTE_B_PIN_NR)
	sbi (TASTE_C_DDR),(TASTE_C_PIN_NR)
	sbi (TASTE_D_DDR),(TASTE_D_PIN_NR)
	sbi (TASTE_E_DDR),(TASTE_E_PIN_NR)
;-------------------------------------------------------------------------------
	tst temp							; TASTE ?	
	breq TASTE_NIX
	sbi (LED_ROT_PORT)  ,(LED_ROT_PIN_NR); ROT  aus	
TASTE_NIX:
;-------------------------------------------------------------------------------
	cpi temp,(TASTE_MENUE_PLUS)+(TASTE_MENUE_MINUS); RESET ?
	breq TASTE_RESET
	ret
;-------------------------------------------------------------------------------
TASTE_RESET:
	call LCD_CLEAR
	call LCD_TEXT_RESET
	call WAIT_200ms	
	call LCD_PUNKT
	call WAIT_200ms	
	call LCD_PUNKT
	call WAIT_200ms	
	call LCD_PUNKT
	call WAIT_200ms	
	jmp RESET
; ##############################################################################
; ##############################################################################
; ##############################################################################
; OUT: CARRY																	
TASTE_PLUS_C:
	LDS temp,(adr_TASTE)
	cpi temp,(TASTE_PLUS)
	breq TASTE_PLUS_RUN
	CLC ;Clear Carry
	ret
;-------------------------------------------------------------------------------
TASTE_PLUS_RUN:
	SEC ;Set Carry
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; OUT: CARRY																	
TASTE_MINUS_C:
	LDS temp,(adr_TASTE)
	cpi temp,(TASTE_MINUS)
	breq TASTE_MINUS_RUN
	CLC ;Clear Carry
	ret
;-------------------------------------------------------------------------------
TASTE_MINUS_RUN:
	SEC ;Set Carry
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
TASTATUR_WAIT:
	LDS temp,(adr_TASTE)
	tst temp
	brne TASTATUR_WAIT_s1
;-------------------------------------------------------------------------------
; ca. 0,5s auf Taste warten														
	ldi ZL, LOW(500)
	ldi ZH,HIGH(500)
TASTATUR_WAIT_s:
	call TASTATUR_ABFRAGE
	LDS temp,(adr_TASTE)
	tst temp
	brne TASTATUR_WAIT_FERTIG
	call wait_1ms
	sbiw ZL,1
	tst ZL
	brne TASTATUR_WAIT_s
	tst ZH
	brne TASTATUR_WAIT_s
TASTATUR_WAIT_FERTIG:
	ret
;-------------------------------------------------------------------------------
; warten, bis Taste losgelassen wird											
TASTATUR_WAIT_s1:
	call WAIT_200ms
	call TASTATUR_ABFRAGE
	LDS temp,(adr_TASTE)
	tst temp
	brne TASTATUR_WAIT_s1
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
PROGRAMM_STOP:
	call TASTATUR_ABFRAGE
	rjmp PROGRAMM_STOP
; ##############################################################################
; ##############################################################################
; ##############################################################################
ZUFALLSZAHL:
	clr ZL	
	clr ZH
	ldi temp9,255
ZUFALLSZAHL_s:
	LD R0,Z+
	add temp,R0
	dec temp9
	brne ZUFALLSZAHL_s
	STS(adr_ZUFALLSZAHL),temp
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
ZAHLER_NULL:
	STS(adr_ZAEHLER0),NULL
	STS(adr_ZAEHLER1),NULL
	STS(adr_ZAEHLER2),NULL
	STS(adr_ZAEHLER3),NULL
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
ZAHLER_PLUS_EINS:
	LDS temp1,(adr_ZAEHLER0)
	LDS temp2,(adr_ZAEHLER1)
	LDS temp3,(adr_ZAEHLER2)
	LDS temp4,(adr_ZAEHLER3)
	add temp1,EINS
	adc temp2,NULL
	adc temp3,NULL
	adc temp4,NULL
	STS(adr_ZAEHLER0),temp1
	STS(adr_ZAEHLER1),temp2
	STS(adr_ZAEHLER2),temp3
	STS(adr_ZAEHLER3),temp4
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
ZAHLER_ANZEIGE:
	LDS temp1,(adr_ZAEHLER0)
	LDS temp2,(adr_ZAEHLER1)
	LDS temp3,(adr_ZAEHLER2)
	LDS temp4,(adr_ZAEHLER3)
	call LCD_ZAHL_TEMP1_2_3_4
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
DATA_CLEAR:
	ldi ZL,LOW (adr_DATA)
	ldi ZH,HIGH(adr_DATA)
	ldi temp9,32 ; 32x16=512
IDE_DATA_CLEAR_s:
	ST Z+,NULL	; 0 
	ST Z+,NULL	; 1 
	ST Z+,NULL	; 2 
	ST Z+,NULL	; 3 
	ST Z+,NULL	; 4 
	ST Z+,NULL	; 5 
	ST Z+,NULL	; 6 
	ST Z+,NULL	; 7 
	ST Z+,NULL	; 8 
	ST Z+,NULL	; 9 
	ST Z+,NULL	; 10
	ST Z+,NULL	; 11
	ST Z+,NULL	; 12
	ST Z+,NULL	; 13
	ST Z+,NULL	; 14
	ST Z+,NULL	; 15
	dec temp9
	brne IDE_DATA_CLEAR_s
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
SECTOR_NULL:
	clr SECTOR0
	clr SECTOR1
	clr SECTOR2
	clr SECTOR3
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
SECTOR_PLUS_EINS:
	add SECTOR0,EINS
	adc SECTOR1,NULL
	adc SECTOR2,NULL
	adc SECTOR3,NULL
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
SECTOR_MINUS_EINS:
	sub SECTOR0,EINS
	sbc SECTOR1,NULL
	sbc SECTOR2,NULL
	sbc SECTOR3,NULL
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
SECTOR_MAL_2:
	LSL SECTOR0					; x2	
	ROL SECTOR1
	ROL SECTOR2
	ROL SECTOR3
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
SECTOR_DURCH_2:
	LSR SECTOR3					; :2	
	ROR SECTOR2
	ROR SECTOR1
	ROR SECTOR0
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
TEMP1_4_NACH_SECTOR:
	MOV SECTOR0,temp1
	MOV SECTOR1,temp2
	MOV SECTOR2,temp3
	MOV SECTOR3,temp4
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
SECTOR_NACH_TEMP1_4:
	MOV temp1,SECTOR0
	MOV temp2,SECTOR1
	MOV temp3,SECTOR2
	MOV temp4,SECTOR3
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
TEMP1_2_3_4_PLUS_EINS:
	add temp1,EINS						; +1		
	adc temp2,NULL
	adc temp3,NULL
	adc temp4,NULL
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
TEMP1_2_3_4_MINUS_EINS:
	sub temp1,EINS						; -1		
	sbc temp2,NULL
	sbc temp3,NULL
	sbc temp4,NULL
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
TEMP1_2_3_4_DURCH_2:
	LSR TEMP4					; :2	
	ROR TEMP3
	ROR TEMP2
	ROR TEMP1
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
TEMP1_2_3_4_MAL_2:
	LSL TEMP1					; x2	
	ROL TEMP2
	ROL TEMP3
	ROL TEMP4
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
TEMP1_2_3_4_MAL_512:
	mov temp4,temp3			; 8xlinks		
	mov temp3,temp2
	mov temp2,temp1
	clr temp1				; LOW			

	LSL temp1				; LOW 1xlinks	
	ROL temp2
	ROL temp3
	ROL temp4				; HIGH			
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
TEMP1_2_3_4_DURCH_512:
	mov temp1,temp2			; 8xlinks		
	mov temp2,temp3
	mov temp3,temp4
	clr temp4
	LSR TEMP4				; :2	
	ROR TEMP3
	ROR TEMP2
	ROR TEMP1
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
TEMP1_2_3_4_DURCH_1000:
	ldi temp5,LOW (1000)
	ldi temp6,HIGH(1000)
	rjmp DIVISION_32_16
;######################################################################## 	
;########################################################################	
;######################################################################## 	
; STAND: 09.12.2011															
;																			
;	Value: 0...	9.999.999.999												
;																			
;	INPUT: 	temp1	(BYTE1)													
;	 		temp2	(BYTE2)													
;			temp3	(BYTE3)													
;			temp4	(BYTE4)													
;																			
;	OUTPUT:	temp1	(		     1-er Stelle)								
;			temp2	(		    10-er Stelle)								
;			temp3	(          100-er Stelle)								
;			temp4	(        1.000-er Stelle)								
;			temp5	(       10.000-er Stelle)								
;			temp6	(      100.000-er Stelle)								
;			temp7	(    1.000.000-er Stelle)								
;			temp8	(   10.000.000-er Stelle)								
;			temp9	(  100.000.000-er Stelle)								
;			temp	(1.000.000.000-er Stelle)								
; 																			
FUNKTION_HEX_ASC_32:
;---------------------------------------------------------------------------
	clr temp			
FUNKTION_HEX_ASC_109:	; 1.100.000.000
	inc temp
	subi temp1, BYTE1    (1000000000)
	sbci temp2, BYTE2    (1000000000)
	sbci temp3, BYTE3    (1000000000)
	sbci temp4, BYTE4    (1000000000)
	brcc FUNKTION_HEX_ASC_109
	subi temp, (-48+1)	; HEX->ASC	
	subi temp1, BYTE1   (-1000000000)
	sbci temp2, BYTE2   (-1000000000)
	sbci temp3, BYTE3   (-1000000000)
	sbci temp4, BYTE4   (-1000000000)
;---------------------------------------------------------------------------
	clr temp9			
FUNKTION_HEX_ASC_108:	; 100.000.000
	inc temp9
	subi temp1, BYTE1    (100000000)
	sbci temp2, BYTE2    (100000000)
	sbci temp3, BYTE3    (100000000)
	sbci temp4, BYTE4    (100000000)
	brcc FUNKTION_HEX_ASC_108
	subi temp9, (-48+1)	; HEX->ASC	
	subi temp1, BYTE1   (-100000000)
	sbci temp2, BYTE2   (-100000000)
	sbci temp3, BYTE3   (-100000000)
	sbci temp4, BYTE4   (-100000000)
;---------------------------------------------------------------------------
	clr temp8
FUNKTION_HEX_ASC_107:	; 10.000.000
	inc temp8
	subi temp1, BYTE1    (10000000)
	sbci temp2, BYTE2    (10000000)
	sbci temp3, BYTE3    (10000000)
	sbci temp4, BYTE4    (10000000)
	brcc FUNKTION_HEX_ASC_107
	subi temp8, (-48+1)	; HEX->ASC	
	subi temp1, BYTE1   (-10000000)
	sbci temp2, BYTE2   (-10000000)
	sbci temp3, BYTE3   (-10000000)
	sbci temp4, BYTE4   (-10000000)
;---------------------------------------------------------------------------
	clr temp7
FUNKTION_HEX_ASC_106:	; 1.000.000
	inc temp7
	subi temp1, BYTE1    (1000000)
	sbci temp2, BYTE2    (1000000)
	sbci temp3, BYTE3    (1000000)
	brcc FUNKTION_HEX_ASC_106
	subi temp7, (-48+1)	; HEX->ASC	
	subi temp1, BYTE1   (-1000000)
	sbci temp2, BYTE2   (-1000000)
	sbci temp3, BYTE3   (-1000000)
;---------------------------------------------------------------------------
	clr temp6
FUNKTION_HEX_ASC_105:	; 100.000	
	inc temp6
	subi temp1, BYTE1    (100000)
	sbci temp2, BYTE2    (100000)
	sbci temp3, BYTE3    (100000)
	brcc FUNKTION_HEX_ASC_105
	subi temp6, (-48+1)	; HEX->ASC	
	subi temp1, BYTE1   (-100000)
	sbci temp2, BYTE2   (-100000)
	sbci temp3, BYTE3   (-100000)
;---------------------------------------------------------------------------	clr temp
	clr temp5
FUNKTION_HEX_ASC_104:	; 10.000	
	inc temp5
	subi temp1, BYTE1    (10000)
	sbci temp2, BYTE2    (10000)
	sbci temp3, BYTE3    (10000)
	brcc FUNKTION_HEX_ASC_104
	subi temp5, (-48+1)	; HEX->ASC	
	subi temp1, BYTE1  (-10000)
	sbci temp2, BYTE2  (-10000)
	sbci temp3, BYTE3  (-10000)
;---------------------------------------------------------------------------
	clr temp4
FUNKTION_HEX_ASC_103:	; 1.000		
	inc temp4
	subi temp1, low  (1000)
	sbci temp2, HIGH (1000)
	brcc FUNKTION_HEX_ASC_103
	subi temp4, (-48+1)	; HEX->ASC	
	subi temp1, low  (-1000)
	sbci temp2, HIGH (-1000)
;---------------------------------------------------------------------------
	clr temp3
FUNKTION_HEX_ASC_102:	; 100		
	inc temp3
	subi temp1, low  (100)
	sbci temp2, HIGH (100)
	brcc FUNKTION_HEX_ASC_102
	subi temp3, (-48+1)	; HEX->ASC	
	subi temp1, low  (-100)
	sbci temp2, HIGH (-100)
;---------------------------------------------------------------------------
	clr temp2
FUNKTION_HEX_ASC_101:	; 10		
	inc temp2
	subi temp1, low  (10)
	brcc FUNKTION_HEX_ASC_101
	subi temp2, (-48+1)	; HEX->ASC	
	subi temp1, low  (-10)
;---------------------------------------------------------------------------
FUNKTION_HEX_ASC_100:	; 1			
	subi temp1, -48		; HEX->ASC	
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; ##############################################################################
; ##############################################################################
; ##############################################################################
; inp: 		temp8 		(BIT-START)	
; inp; 		temp9		(BIT_ENDE)	
; ZL/ZH:	SRAM					
; OUT: 		temp1..5				
; Hilfsregister: R0, R1				
FUNKTION_SRAM_SELECT_Z_8_9:
	clr temp1
	clr temp2
	clr temp3
	clr temp4
	clr temp5
	ldi temp,0b10000000
	mov R1,temp			; Hilfsregister	
	inc temp9
	sub temp9,temp8		; Schleife		
FUNK_SRAM_s:
	mov temp,temp8		; (BIT-NR)		
	andi temp,0b11111000
	LSR temp			; rechts ohne C	
	LSR temp
	LSR temp
	push ZL				; Z sichern		
	push ZH
	sub ZL,temp			; subtrahieren	
	sbc ZH,NULL
	LD R0,Z				; load			
	pop ZH				; Z wiederherstellen
	pop ZL
; ------------------------------------------------------------------------------
	mov temp,temp8		; (BIT-NR)			
	andi temp,0b00000111
	cpi temp,0
	breq FUNK_SRAM0
	cpi temp,1
	breq FUNK_SRAM1
	cpi temp,2
	breq FUNK_SRAM2
	cpi temp,3
	breq FUNK_SRAM3
	cpi temp,4
	breq FUNK_SRAM4
	cpi temp,5
	breq FUNK_SRAM5
	cpi temp,6
	breq FUNK_SRAM6
	rjmp FUNK_SRAM7
; ------------------------------------------------------------------------------
FUNK_SRAM0:
	CLC				; Clear C	
	SBRC R0,0
	SEC 			; Set C		
	rjmp FUNK_SRAM_w
FUNK_SRAM1:
	CLC				; Clear C	
	SBRC R0,1
	SEC 			; Set C		
	rjmp FUNK_SRAM_w
FUNK_SRAM2:
	CLC				; Clear C	
	SBRC R0,2
	SEC 			; Set C		
	rjmp FUNK_SRAM_w
FUNK_SRAM3:
	CLC				; Clear C	
	SBRC R0,3
	SEC 			; Set C		
	rjmp FUNK_SRAM_w
FUNK_SRAM4:
	CLC				; Clear C	
	SBRC R0,4
	SEC 			; Set C		
	rjmp FUNK_SRAM_w
FUNK_SRAM5:
	CLC				; Clear C	
	SBRC R0,5
	SEC 			; Set C		
	rjmp FUNK_SRAM_w
FUNK_SRAM6:
	CLC				; Clear C	
	SBRC R0,6
	SEC 			; Set C		
	rjmp FUNK_SRAM_w
FUNK_SRAM7:
	CLC				; Clear C	
	SBRC R0,7
	SEC 			; Set C		
	rjmp FUNK_SRAM_w
FUNK_SRAM_w:
	rcall FUNK_SRAM_RECHTS_C	; rechts ber C
	inc temp8		; (BIT-NR)	
	dec temp9		; Schleife	
	breq FUNK_SRAM_w2
	rjmp FUNK_SRAM_s
; ------------------------------------------------------------------------------
FUNK_SRAM_w2:
	SBRC temp1,0	
	rjmp FUNK_SRAM_w3
	CLC				; Clear C	
	rcall FUNK_SRAM_RECHTS_C	; rechts ber C
	rjmp FUNK_SRAM_w2
; ------------------------------------------------------------------------------
FUNK_SRAM_w3:
	CLC				; Clear C	
	rcall FUNK_SRAM_RECHTS_C	; rechts ber C
	ret
; ------------------------------------------------------------------------------
FUNK_SRAM_RECHTS_C:
	ROR R1			; rechts ber C
	ROR temp5
	ROR temp4
	ROR temp3
	ROR temp2
	ROR temp1
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; Version: 14.11.2014																	
;																						
; 	INPUT:	TEMP1 / TEMP2 	1.Faktor													
;			TEMP3 / TEMP4	2.Faktor													
; 																						
; 	OUTPUT: TEMP1...4	Produkt															
; 																						
;	HILFSREGISTER: 	temp5+6+7+8 (werden gesichert)										
; 																						
MULTIPLIKATION_16_16_32:
	push temp5
	push temp6
	push temp7
	push temp8

	clr	r2
	mul	temp4,temp2		; ah * bh
	mov	temp7,r0
	mov	temp8,r1
	mul	temp3,temp1		; al * bl
	mov	temp5,r0
	mov	temp6,r1
	mul	temp4,temp1		; ah * bl
	add	temp6,r0
	adc	temp7,r1
	adc	temp8,r2
	mul	temp2,temp3		; bh * al
	add	temp6,r0
	adc	temp7,r1
	adc	temp8,r2


	mov temp1,temp5
	mov temp2,temp6
	mov temp3,temp7
	mov temp4,temp8

	pop temp8
	pop temp7
	pop temp6
	pop temp5

ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; Version: 11/2014																
;																				
;	Division 32 bit-Zahl durch 16 bit Zahl										
;																				
;	input/Output 	temp1 (LOW)	(0...4.294.967.295)								
;	input/Output 	temp2														
;	input/Output 	temp3														
;	input/Output 	temp4 (HIGH)												
;																				
;	input	temp5	Divisor	(LOW)	 											
;	input	temp6	Divisor	(HIGH)	 											
;																				
; Register:	R0+R1	Hilfsregister 												
;			temp	Hilfsregister 												
;																				
DIVISION_32_16:
	ldi     temp,32    ;32* schieben	
    clr     R0          ;Rest =0		
    clr     R1
DIVISION_32_16_1:    
	lsl     temp1       ;Ein Bit aus dem Dividend ...   
    rol     temp2       ;ins Carry schieben
    rol     temp3
    rol     temp4
    rol     R0          ;Carry in den Rest schieben
    rol     R1
    brcs    DIVISION_32_16_3       ;Bei berlauf von Rest, Sprung
    cp      R0,temp5       					;Solange Bits in temp1 (Rest) schieben bis >= divisor
    cpc     R1,temp6
    brcs    DIVISION_32_16_2    	;Bei Rest<Divisor, die eingeschobene 0 in temp1 stehen lassen
DIVISION_32_16_3:
    inc     temp1       ;Ansonsten Bit0 setzen und den Divisor vom Rest abziehen
    sub     R0,temp5
    sbc     R1,temp6
DIVISION_32_16_2:
   	dec     temp       ;Das ganze 32 mal
    brne    DIVISION_32_16_1
ret 
; ##############################################################################
; ##############################################################################
; ##############################################################################

