; ##############################################################################
; ##############################################################################
; ##############################################################################
; SPCR => SPIE SPE DORD MSTR CPOL CPHA SPR1 SPR0 (SPI Control Register)			
; 																				
; SPI Mode Conditions Leading Edge Trailing eDge								
; 0 CPOL=0, CPHA=0 Sample (Rising) 	Setup (Falling)								
; 1 CPOL=0, CPHA=1 Setup (Rising) 	Sample (Falling)							
; 2 CPOL=1, CPHA=0 Sample (Falling) Setup (Rising)								
; 3 CPOL=1, CPHA=1 Setup (Falling) 	Sample (Rising)								
; 																				
; SPI2X SPR1 SPR0 	SCK Frequency												
; 0 	0 	 0 		fosc/4														
; 0 	0 	 1 		fosc/16														
; 0 	1 	 0 		fosc/64														
; 0 	1 	 1 		fosc/128													
; 1 	0 	 0 		fosc/2														
; 1 	0 	 1 		fosc/8														
; 1 	1 	 0 		fosc/32														
; 1 	1 	 1 		fosc/64														
; 																				
SPI_INITIALISIERUNG:
	rcall SPI_CS_HIGH				 		; HIGH (LOW=aktiv)
	; PORTS									
	sbi (SPI_CS_DDR)  ,(SPI_CS_PIN_NR)		; PORT auf Ausgang	
	sbi (SPI_MOSI_DDR),(SPI_MOSI_PIN_NR)	; PORT auf Ausgang	
	sbi (SPI_SCK_DDR) ,(SPI_SCK_PIN_NR)		; PORT auf Ausgang	
	cbi (SPI_MISO_DDR),(SPI_MISO_PIN_NR)	; PORT auf EINGANG	
	sbi (SPI_MISO_PORT),(SPI_MISO_PIN_NR)	; Pull-UP on		
	call wait_500ms							; diese PAUSE ist wichtig		
;	call SPI_TAKT_LOW
	call SPI_TAKT_HIGH
;	call SPI_TAKT_SUPER_HIGH
ret
; ##############################################################################
; fosc/128																		
SPI_TAKT_LOW:
	ldi temp,(0<<SPIE)|(1<<SPE)|(0<<DORD)|(1<<MSTR)|(0<<CPOL)|(0<<CPHA)|(1<<SPR1)|(1<<SPR0)
	out SPCR,temp
	ldi temp,(0<<SPI2X)
	out SPSR,temp
ret
; ##############################################################################
; fosc/4																		
SPI_TAKT_HIGH:
	ldi temp,(0<<SPIE)|(1<<SPE)|(0<<DORD)|(1<<MSTR)|(0<<CPOL)|(0<<CPHA)|(0<<SPR1)|(0<<SPR0)
	out SPCR,temp
	ldi temp,(0<<SPI2X)
	out SPSR,temp
ret
; ##############################################################################
; fosc/2																		
SPI_TAKT_SUPER_HIGH:
	ldi temp,(0<<SPIE)|(1<<SPE)|(0<<DORD)|(1<<MSTR)|(0<<CPOL)|(0<<CPHA)|(0<<SPR1)|(0<<SPR0)
	out SPCR,temp
	ldi temp,(1<<SPI2X)
	out SPSR,temp
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
SPI_CS_HIGH:
	sbi(SPI_CS_PORT),(SPI_CS_PIN_NR)		; CS auf HIGH	
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
SPI_CS_LOW:
	cbi  (SPI_CS_PORT),(SPI_CS_PIN_NR)		; CS auf LOW !!!
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; inp: TEMP																		
; OUT: TEMP																		
SPI_TX_RX_TEMP:
	out SPDR,temp			; Start transmission of data 	
	WDR
SPI_TX_RX_TEMP_w:
	in temp,SPSR
	sbrs temp,SPIF
	rjmp SPI_TX_RX_TEMP_w	; Wait for transmission complete
	in temp, SPDR			; DATENBYTE EINLESEN			
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; TX:  0xFF	(DUMMY)																
; OUT: temp																		
SPI_RX_TEMP:
	out SPDR,VOLL			; Start transmission of data 	; DUMMY (xFF)					
	WDR
SPI_RX_TEMP_w:
	in temp,SPSR
	sbrs temp,SPIF
	rjmp SPI_RX_TEMP_w		; Wait for transmission complete
	in temp, SPDR			; DATENBYTE EINLESEN			
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; 512 Bytes SD--->SRAM															
SPI_RX512_DATA:
	ldi ZL, LOW(adr_DATA)		; Z-ZEIGER initialisieren	
	ldi ZH,HIGH(adr_DATA)
	ldi temp9,32				; Schleife (32x16=512)		
SPI_RX512_DATA_s:
	rcall SPI_RX_TEMP			; OUT: temp
	ST Z+,temp					; speichern und Z+1	
	rcall SPI_RX_TEMP			; OUT: temp
	ST Z+,temp					; speichern und Z+1	
	rcall SPI_RX_TEMP			; OUT: temp
	ST Z+,temp					; speichern und Z+1	
	rcall SPI_RX_TEMP			; OUT: temp
	ST Z+,temp					; speichern und Z+1	
	rcall SPI_RX_TEMP			; OUT: temp
	ST Z+,temp					; speichern und Z+1	
	rcall SPI_RX_TEMP			; OUT: temp
	ST Z+,temp					; speichern und Z+1	
	rcall SPI_RX_TEMP			; OUT: temp
	ST Z+,temp					; speichern und Z+1	
	rcall SPI_RX_TEMP			; OUT: temp
	ST Z+,temp					; speichern und Z+1	
	rcall SPI_RX_TEMP			; OUT: temp
	ST Z+,temp					; speichern und Z+1	
	rcall SPI_RX_TEMP			; OUT: temp
	ST Z+,temp					; speichern und Z+1	
	rcall SPI_RX_TEMP			; OUT: temp
	ST Z+,temp					; speichern und Z+1	
	rcall SPI_RX_TEMP			; OUT: temp
	ST Z+,temp					; speichern und Z+1	
	rcall SPI_RX_TEMP			; OUT: temp
	ST Z+,temp					; speichern und Z+1	
	rcall SPI_RX_TEMP			; OUT: temp
	ST Z+,temp					; speichern und Z+1	
	rcall SPI_RX_TEMP			; OUT: temp
	ST Z+,temp					; speichern und Z+1	
	rcall SPI_RX_TEMP			; OUT: temp
	ST Z+,temp					; speichern und Z+1	
	dec temp9
	brne SPI_RX512_DATA_s
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; 512 Bytes SRAM--->SD															
SPI_TX512_DATA:
	ldi ZL, LOW(adr_DATA)		; Z-ZEIGER initialisieren	
	ldi ZH,HIGH(adr_DATA)
	ldi temp9,32					; Schleife (512)		
SD_SEKTOR_WRITE_DATA:
	LD temp,Z+						; DATA				
	rcall SPI_TX_RX_TEMP			; INP/OUT: TEMP		
	LD temp,Z+						; DATA				
	rcall SPI_TX_RX_TEMP			; INP/OUT: TEMP		
	LD temp,Z+						; DATA				
	rcall SPI_TX_RX_TEMP			; INP/OUT: TEMP		
	LD temp,Z+						; DATA				
	rcall SPI_TX_RX_TEMP			; INP/OUT: TEMP		
	LD temp,Z+						; DATA				
	rcall SPI_TX_RX_TEMP			; INP/OUT: TEMP		
	LD temp,Z+						; DATA				
	rcall SPI_TX_RX_TEMP			; INP/OUT: TEMP		
	LD temp,Z+						; DATA				
	rcall SPI_TX_RX_TEMP			; INP/OUT: TEMP		
	LD temp,Z+						; DATA				
	rcall SPI_TX_RX_TEMP			; INP/OUT: TEMP		
	LD temp,Z+						; DATA				
	rcall SPI_TX_RX_TEMP			; INP/OUT: TEMP		
	LD temp,Z+						; DATA				
	rcall SPI_TX_RX_TEMP			; INP/OUT: TEMP		
	LD temp,Z+						; DATA				
	rcall SPI_TX_RX_TEMP			; INP/OUT: TEMP		
	LD temp,Z+						; DATA				
	rcall SPI_TX_RX_TEMP			; INP/OUT: TEMP		
	LD temp,Z+						; DATA				
	rcall SPI_TX_RX_TEMP			; INP/OUT: TEMP		
	LD temp,Z+						; DATA				
	rcall SPI_TX_RX_TEMP			; INP/OUT: TEMP		
	LD temp,Z+						; DATA				
	rcall SPI_TX_RX_TEMP			; INP/OUT: TEMP		
	LD temp,Z+						; DATA				
	rcall SPI_TX_RX_TEMP			; INP/OUT: TEMP		
	dec  temp9						; Schleife			
	brne SD_SEKTOR_WRITE_DATA
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; max 1s																		
; auf  RX=temp warten															
; OUT: Carry																	
SPI_WAIT_TEMP1_C:
	push ZL
	push ZH
	ldi ZL,LOW (10000)		; Schleife			
	LDI ZH,HIGH(10000)
; ------------------------------------------------------------------------------
SPI_WAIT_TEMP1_s:
	rcall SPI_RX_TEMP		; OUT: temp	
	cp temp,temp1
	breq SPI_WAIT_TEMP_OK
	rcall SPI_RX_TEMP		; OUT: temp	
	cp temp,temp1
	breq SPI_WAIT_TEMP_OK
	rcall SPI_RX_TEMP		; OUT: temp	
	cp temp,temp1
	breq SPI_WAIT_TEMP_OK
	rcall SPI_RX_TEMP		; OUT: temp	
	cp temp,temp1
	breq SPI_WAIT_TEMP_OK
	rcall SPI_RX_TEMP		; OUT: temp	
	cp temp,temp1
	breq SPI_WAIT_TEMP_OK
	rcall SPI_RX_TEMP		; OUT: temp	
	cp temp,temp1
	breq SPI_WAIT_TEMP_OK
	rcall SPI_RX_TEMP		; OUT: temp	
	cp temp,temp1
	breq SPI_WAIT_TEMP_OK
	rcall SPI_RX_TEMP		; OUT: temp	
	cp temp,temp1
	breq SPI_WAIT_TEMP_OK
; ------------------------------------------------------------------------------
	sbiw ZL,1				; -1
	tst ZL
	brne SPI_WAIT_TEMP1_s
	tst ZH
	brne SPI_WAIT_TEMP1_s
; ------------------------------------------------------------------------------
SPI_WAIT_TEMP_ERROR:
	pop ZH
	pop ZL
	CLC 					; Clear Carry	
	ret
; ------------------------------------------------------------------------------
SPI_WAIT_TEMP_OK:
	pop ZH
	pop ZL
	SEC 					; Set Carry		
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; auf  3x auf 0xFF warten, dann CS_HIGH											
SPI_WAIT_FF_HIGH:
	rcall SPI_RX_TEMP		; OUT: temp
	cpi temp,0xFF
	brne SPI_WAIT_FF_HIGH
	rcall SPI_RX_TEMP		; OUT: temp
	cpi temp,0xFF
	brne SPI_WAIT_FF_HIGH
	rcall SPI_RX_TEMP		; OUT: temp
	cpi temp,0xFF
	brne SPI_WAIT_FF_HIGH
	rjmp SPI_CS_HIGH		; CS auf HIGH		
; ##############################################################################
; ##############################################################################
; ##############################################################################
SPI_CLEAR_HIGH:
	rcall SPI_CLEAR255
	rcall SPI_CLEAR255
	rcall SPI_CLEAR255
	rcall SPI_CLEAR255
	rcall SPI_CS_HIGH		; CS auf HIGH		
	rcall SPI_CLEAR255
ret
; ------------------------------------------------------------------------------
SPI_CLEAR255:
	push temp9
	ldi temp9,255
SPI_CLEAR_s:
	rcall SPI_RX_TEMP			; OUT: temp
	dec temp9
	brne SPI_CLEAR_s
	pop temp9
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
SPI_DEBUG_ANZEIGE:
	call LCD_CLEAR
	call LCD_ZEILE1	; ZEILE1	
	call SPI_DEBUG_ZEILE
	call LCD_ZEILE2	; ZEILE2	
	call SPI_DEBUG_ZEILE
	call LCD_ZEILE3	; ZEILE3	
	call SPI_DEBUG_ZEILE
	call LCD_ZEILE4	; ZEILE4	
	call SPI_DEBUG_ZEILE

;	call WAIT_100ms
	call WAIT_1s
	call PROGRAMM_STOP	
ret
; ------------------------------------------------------------------------------
SPI_DEBUG_ZEILE:
	push temp1
	push temp9
	ldi temp9,8
SPI_DEBUG_s:
	WDR
	rcall SPI_RX_TEMP		; OUT: temp
	mov temp1,temp
	call LCD_HEX_TEMP1
	dec temp9
	brne SPI_DEBUG_s
	pop temp9
	pop temp1
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################


